import * as pqb from 'pqb';
import { ColumnsShape, ColumnType, EnumColumn, ArrayColumn, JSONColumn, JSONTypeAny, JSONArray, JSONEnum, JSONInstanceOf, JSONLiteral, JSONMap, JSONSet, JSONNativeEnum, JSONTuple, JSONObject, UnknownKeysParam, JSONRecord, JSONRecordKeyType, JSONIntersection, JSONDiscriminatedUnion, Primitive } from 'pqb';
import { z } from 'zod';
import { Buffer } from 'node:buffer';

declare type NumberType = 'smallint' | 'integer' | 'real' | 'smallserial' | 'serial' | 'money';
declare type BigIntType = 'bigint' | 'numeric' | 'decimal' | 'double precision' | 'bigserial';
declare type StringType = 'varchar' | 'char' | 'text' | 'string' | 'xml' | 'json';
declare type DateTimeType = 'date' | 'timestamp' | 'timestamp with time zone';
declare type TimeType = 'time' | 'time with time zone';
declare type GeometryType = 'point' | 'line' | 'lseg' | 'box' | 'path' | 'polygon' | 'circle';
declare type NetworkType = 'cidr' | 'inet' | 'macaddr' | 'macaddr8';
declare type BitStringType = 'bit' | 'bit varying';
declare type FullTextSearchType = 'tsvector' | 'tsquery';
declare type UUIDType = 'uuid';
declare type ByteaType = 'bytea';
declare type SchemaToZod<T extends ColumnType, D = T['dataType']> = T['isNullable'] extends true ? z.ZodNullable<SchemaToZod<Omit<T, 'isNullable'> & {
    isNullable: false;
}>> : D extends NumberType ? z.ZodNumber : D extends BigIntType | StringType | TimeType | GeometryType | NetworkType | BitStringType | FullTextSearchType | UUIDType ? z.ZodString : D extends ByteaType ? z.ZodType<Buffer> : D extends DateTimeType ? z.ZodDate : D extends 'interval' ? typeof interval : D extends 'boolean' ? z.ZodBoolean : T extends EnumColumn<string, infer U> ? z.ZodEnum<U> : T extends ArrayColumn<infer U> ? z.ZodArray<SchemaToZod<U>> : T extends JSONColumn ? JsonToZod<T['data']['schema']> : never;
declare type JsonToZod<T extends JSONTypeAny, D = T['dataType']> = T extends {
    types: [JSONTypeAny, JSONTypeAny, ...JSONTypeAny[]];
} ? z.ZodUnion<MapJsonTuple<T['types']>> : T['data'] extends {
    nullable: true;
} ? T['data'] extends {
    optional: true;
} ? z.ZodNullable<z.ZodOptional<JsonToZod<Omit<T, 'data'> & {
    data: Omit<T['data'], 'nullable' | 'optional'>;
}>>> : z.ZodNullable<JsonToZod<Omit<T, 'data'> & {
    data: Omit<T['data'], 'nullable'>;
}>> : T['data'] extends {
    optional: true;
} ? z.ZodOptional<JsonToZod<Omit<T, 'data'> & {
    data: Omit<T['data'], 'optional'>;
}>> : D extends 'bigint' ? z.ZodString : D extends 'boolean' ? z.ZodBoolean : D extends 'date' ? z.ZodDate : D extends 'nan' ? z.ZodNaN : D extends 'never' ? z.ZodNever : D extends 'null' ? z.ZodNull : D extends 'number' ? z.ZodNumber : D extends 'string' ? z.ZodString : D extends 'undefined' ? z.ZodUndefined : D extends 'unknown' ? z.ZodUnknown : D extends 'void' ? z.ZodVoid : T extends JSONArray<infer U> ? z.ZodArray<JsonToZod<U>> : T extends JSONEnum<string, infer U> ? z.ZodEnum<U> : T extends JSONInstanceOf<infer U> ? z.ZodType<InstanceType<U>, z.ZodTypeDef, InstanceType<U>> : T extends JSONLiteral<infer U> ? z.ZodLiteral<U> : T extends JSONMap<infer K, infer V> ? z.ZodMap<JsonToZod<K>, JsonToZod<V>> : T extends JSONSet<infer U> ? z.ZodSet<JsonToZod<U>> : T extends JSONNativeEnum<infer U> ? z.ZodNativeEnum<U> : T extends JSONTuple<infer U, infer R> ? z.ZodTuple<MapJsonTuple<U>, R extends JSONTypeAny ? JsonToZod<R> : null> : T extends JSONObject<Record<string, JSONTypeAny>, UnknownKeysParam> ? z.ZodObject<{
    [K in keyof T['shape']]: JsonToZod<T['shape'][K]>;
}, T['unknownKeys'], JsonToZod<T['catchAllType']>> : T extends JSONRecord<JSONRecordKeyType, JSONTypeAny> ? z.ZodRecord<JsonToZod<T['keyType']> extends z.ZodType<string | number | symbol, any, any> ? JsonToZod<T['keyType']> : never, JsonToZod<T['valueType']>> : T extends JSONIntersection<JSONTypeAny, JSONTypeAny> ? z.ZodIntersection<JsonToZod<T['left']>, JsonToZod<T['right']>> : T extends JSONDiscriminatedUnion<string, Primitive, any> ? z.ZodDiscriminatedUnion<T['discriminator'], T['discriminatorValue'], JsonToZod<T['_option']> extends z.ZodDiscriminatedUnionOption<T['discriminator'], T['discriminatorValue']> ? JsonToZod<T['_option']> : never> : D extends 'any' ? z.ZodTypeAny : z.ZodType<T['type']>;
declare type MapJsonTuple<T extends unknown[]> = T extends [infer Head, ...infer Tail] ? [Head extends JSONTypeAny ? JsonToZod<Head> : never, ...MapJsonTuple<Tail>] : [];
declare type ModelToZod<T extends new () => {
    columns: {
        shape: ColumnsShape;
    };
}> = InstanceToZod<InstanceType<T>['columns']>;
declare const modelToZod: <T extends new () => {
    columns: {
        shape: ColumnsShape;
    };
}>(model: T) => ModelToZod<T>;
declare type InstanceToZod<T extends {
    shape: ColumnsShape;
}> = z.ZodObject<{
    [K in keyof T['shape']]: SchemaToZod<T['shape'][K]>;
}>;
declare const instanceToZod: <T extends {
    shape: ColumnsShape;
}>({ shape, }: T) => InstanceToZod<T>;
declare const columnToZod: <T extends ColumnType<unknown, pqb.Operators, unknown>>(column: T) => SchemaToZod<T, T["dataType"]>;
declare const interval: z.ZodObject<{
    years: z.ZodOptional<z.ZodNumber>;
    months: z.ZodOptional<z.ZodNumber>;
    days: z.ZodOptional<z.ZodNumber>;
    hours: z.ZodOptional<z.ZodNumber>;
    seconds: z.ZodOptional<z.ZodNumber>;
}, "strict", z.ZodTypeAny, {
    years?: number | undefined;
    months?: number | undefined;
    days?: number | undefined;
    hours?: number | undefined;
    seconds?: number | undefined;
}, {
    years?: number | undefined;
    months?: number | undefined;
    days?: number | undefined;
    hours?: number | undefined;
    seconds?: number | undefined;
}>;

export { InstanceToZod, ModelToZod, columnToZod, instanceToZod, modelToZod };
